# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.user_info import UserInfo
from ..types.user_retrieve_response import UserRetrieveResponse

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        user_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UserRetrieveResponse:
        """Get basic information for a user.

        The requester must belong to the same
        organization as the target user and be an admin or manager of an account the
        target user belongs to.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            f"/users/{user_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserRetrieveResponse,
        )

    def info(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UserInfo:
        """Get basic information for the logged in user making this request."""
        return self._get(
            "/user-info",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserInfo,
        )


class AsyncUsersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        user_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UserRetrieveResponse:
        """Get basic information for a user.

        The requester must belong to the same
        organization as the target user and be an admin or manager of an account the
        target user belongs to.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            f"/users/{user_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserRetrieveResponse,
        )

    async def info(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> UserInfo:
        """Get basic information for the logged in user making this request."""
        return await self._get(
            "/user-info",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserInfo,
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve = to_raw_response_wrapper(
            users.retrieve,
        )
        self.info = to_raw_response_wrapper(
            users.info,
        )


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve = async_to_raw_response_wrapper(
            users.retrieve,
        )
        self.info = async_to_raw_response_wrapper(
            users.info,
        )


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.retrieve = to_streamed_response_wrapper(
            users.retrieve,
        )
        self.info = to_streamed_response_wrapper(
            users.info,
        )


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.retrieve = async_to_streamed_response_wrapper(
            users.retrieve,
        )
        self.info = async_to_streamed_response_wrapper(
            users.info,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .._models import BaseModel

__all__ = ["HybridEvaluationMetrics", "AllQuestionInfo"]


class AllQuestionInfo(BaseModel):
    agreement_rate: float

    evaluation_question: str

    is_conditional_question: bool

    num_correct: int

    num_free_text: int

    num_inconclusive: int

    num_wrong: int

    question_title: str

    question_type: str

    total_time: float

    total_tokens: float

    choices: Optional[List[object]] = None


class HybridEvaluationMetrics(BaseModel):
    all_question_info: Dict[str, AllQuestionInfo]
    """Information about all questions."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SlackDataSourceConfig"]


class SlackDataSourceConfig(BaseModel):
    channel_id: str
    """Slack Channel or Conversation ID to retrieve history from.

    Open channel details and find the ID at bottom of 'About' section.
    """

    source: Literal["Slack"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .parameter_schema_field_param import ParameterSchemaFieldParam

__all__ = ["ParameterSchemaParam"]


class ParameterSchemaParam(TypedDict, total=False):
    parameters: Required[Iterable[ParameterSchemaFieldParam]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.knowledge_bases import artifact_list_params, artifact_retrieve_params
from ...types.knowledge_bases.artifact import Artifact
from ...types.knowledge_bases.artifact_list_response import ArtifactListResponse

__all__ = ["ArtifactsResource", "AsyncArtifactsResource"]


class ArtifactsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ArtifactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ArtifactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ArtifactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ArtifactsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        artifact_id: str,
        *,
        knowledge_base_id: str,
        status_filter: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Artifact:
        """
        ### Description

        Gets the details of an artifact tracked by a knowledge base.

        ### Details

        This API can be used to get information about a single artifact by ID. This
        response will contain much more detail about the artifact than show in the
        [List Artifacts API](https://scale-egp.readme.io/reference/list_knowledge_base_artifacts_v2)
        call. To use this API, pass in the `knowledge_base_id` and `artifact_id` that
        were returned from your
        [List Artifacts API](https://scale-egp.readme.io/reference/list_knowledge_base_artifacts_v2)
        call as path parameters.

        #### Compatibility with V1

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not artifact_id:
            raise ValueError(f"Expected a non-empty value for `artifact_id` but received {artifact_id!r}")
        return self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}/artifacts/{artifact_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"status_filter": status_filter}, artifact_retrieve_params.ArtifactRetrieveParams
                ),
            ),
            cast_to=Artifact,
        )

    def list(
        self,
        knowledge_base_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        status: Literal["Pending", "Completed", "Failed", "Uploading", "Deleting"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ArtifactListResponse]:
        """
        ### Description

        List all artifacts tracked by a knowledge base.

        ### Details

        This API can be used to list all artifacts that are currently tracked in a
        knowledge base. This API will return the details of all artifacts including
        their IDs, names, the source they originated from, their current upload
        statuses, and the timestamps for their creation and last-updated time.

        This list should be consistent with the state of the data source at the time of
        start of the latest upload. If the state is not consistent, create a new upload
        to update the knowledge base to reflect the latest state of the data source.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          status: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/artifacts",
            page=SyncPageResponse[ArtifactListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                        "status": status,
                    },
                    artifact_list_params.ArtifactListParams,
                ),
            ),
            model=ArtifactListResponse,
        )


class AsyncArtifactsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncArtifactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncArtifactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncArtifactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncArtifactsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        artifact_id: str,
        *,
        knowledge_base_id: str,
        status_filter: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Artifact:
        """
        ### Description

        Gets the details of an artifact tracked by a knowledge base.

        ### Details

        This API can be used to get information about a single artifact by ID. This
        response will contain much more detail about the artifact than show in the
        [List Artifacts API](https://scale-egp.readme.io/reference/list_knowledge_base_artifacts_v2)
        call. To use this API, pass in the `knowledge_base_id` and `artifact_id` that
        were returned from your
        [List Artifacts API](https://scale-egp.readme.io/reference/list_knowledge_base_artifacts_v2)
        call as path parameters.

        #### Compatibility with V1

        V2 and V1 Knowledge Bases are entirely separate and not backwards compatible.
        Users who have existing V1 knowledge bases will need to migrate their data to V2
        knowledge bases.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not artifact_id:
            raise ValueError(f"Expected a non-empty value for `artifact_id` but received {artifact_id!r}")
        return await self._get(
            f"/v4/knowledge-bases/{knowledge_base_id}/artifacts/{artifact_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"status_filter": status_filter}, artifact_retrieve_params.ArtifactRetrieveParams
                ),
            ),
            cast_to=Artifact,
        )

    def list(
        self,
        knowledge_base_id: str,
        *,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        status: Literal["Pending", "Completed", "Failed", "Uploading", "Deleting"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ArtifactListResponse, AsyncPageResponse[ArtifactListResponse]]:
        """
        ### Description

        List all artifacts tracked by a knowledge base.

        ### Details

        This API can be used to list all artifacts that are currently tracked in a
        knowledge base. This API will return the details of all artifacts including
        their IDs, names, the source they originated from, their current upload
        statuses, and the timestamps for their creation and last-updated time.

        This list should be consistent with the state of the data source at the time of
        start of the latest upload. If the state is not consistent, create a new upload
        to update the knowledge base to reflect the latest state of the data source.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          status: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/artifacts",
            page=AsyncPageResponse[ArtifactListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "page": page,
                        "status": status,
                    },
                    artifact_list_params.ArtifactListParams,
                ),
            ),
            model=ArtifactListResponse,
        )


class ArtifactsResourceWithRawResponse:
    def __init__(self, artifacts: ArtifactsResource) -> None:
        self._artifacts = artifacts

        self.retrieve = to_raw_response_wrapper(
            artifacts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            artifacts.list,
        )


class AsyncArtifactsResourceWithRawResponse:
    def __init__(self, artifacts: AsyncArtifactsResource) -> None:
        self._artifacts = artifacts

        self.retrieve = async_to_raw_response_wrapper(
            artifacts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            artifacts.list,
        )


class ArtifactsResourceWithStreamingResponse:
    def __init__(self, artifacts: ArtifactsResource) -> None:
        self._artifacts = artifacts

        self.retrieve = to_streamed_response_wrapper(
            artifacts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            artifacts.list,
        )


class AsyncArtifactsResourceWithStreamingResponse:
    def __init__(self, artifacts: AsyncArtifactsResource) -> None:
        self._artifacts = artifacts

        self.retrieve = async_to_streamed_response_wrapper(
            artifacts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            artifacts.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ExecuteAgentResponse", "Context", "ContextToolRequest"]


class ContextToolRequest(BaseModel):
    arguments: str
    """Arguments to pass to the tool.

    The format must be a JSON Schema-compliant object serialized into a string.
    """

    name: str
    """Name of the tool that the AI wants the client to use."""


class Context(BaseModel):
    content: Optional[str] = None
    """The final output of the agent when it no longer needs any tools"""

    tool_request: Optional[ContextToolRequest] = None
    """The tool request if the agent needs more information."""


class ExecuteAgentResponse(BaseModel):
    action: Literal["tool_request", "content"]
    """An enumeration."""

    context: Context
    """Context object containing the output payload.

    This will contain a key for all actions that the agent can perform. However,
    only the key corresponding to the action that the agent performed have a
    populated value. The rest of the values will be `null`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncTopLevelArray, AsyncTopLevelArray
from ...._base_client import AsyncPaginator, make_request_options
from ....types.chat_thread import ChatThread

__all__ = ["ChatThreadsResource", "AsyncChatThreadsResource"]


class ChatThreadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatThreadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ChatThreadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatThreadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ChatThreadsResourceWithStreamingResponse(self)

    def list(
        self,
        application_variant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncTopLevelArray[ChatThread]:
        """
        List Application Threads

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return self._get_api_list(
            f"/v4/applications/{application_variant_id}/threads",
            page=SyncTopLevelArray[ChatThread],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ChatThread,
        )


class AsyncChatThreadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatThreadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChatThreadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatThreadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncChatThreadsResourceWithStreamingResponse(self)

    def list(
        self,
        application_variant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ChatThread, AsyncTopLevelArray[ChatThread]]:
        """
        List Application Threads

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_variant_id:
            raise ValueError(
                f"Expected a non-empty value for `application_variant_id` but received {application_variant_id!r}"
            )
        return self._get_api_list(
            f"/v4/applications/{application_variant_id}/threads",
            page=AsyncTopLevelArray[ChatThread],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=ChatThread,
        )


class ChatThreadsResourceWithRawResponse:
    def __init__(self, chat_threads: ChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.list = to_raw_response_wrapper(
            chat_threads.list,
        )


class AsyncChatThreadsResourceWithRawResponse:
    def __init__(self, chat_threads: AsyncChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.list = async_to_raw_response_wrapper(
            chat_threads.list,
        )


class ChatThreadsResourceWithStreamingResponse:
    def __init__(self, chat_threads: ChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.list = to_streamed_response_wrapper(
            chat_threads.list,
        )


class AsyncChatThreadsResourceWithStreamingResponse:
    def __init__(self, chat_threads: AsyncChatThreadsResource) -> None:
        self._chat_threads = chat_threads

        self.list = async_to_streamed_response_wrapper(
            chat_threads.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ChunkListResponse"]


class ChunkListResponse(BaseModel):
    chunk_id: str
    """The unique ID of the chunk with embedding"""

    text: str
    """The text associated with the chunk"""

    attachment_url: Optional[str] = None
    """Original attachment URL from which this chunk got its data from"""

    embedding: Optional[List[float]] = None
    """The vector embedding of the text associated with the chunk"""

    metadata: Optional[object] = None
    """
    Any additional key value pairs of information stored by you on the chunk with
    embedding
    """

    title: Optional[str] = None
    """Title for this chunk, for example the file name"""

    user_supplied_metadata: Optional[object] = None
    """
    Any additional key value pairs of information returned from the custom chunking.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, overload
from typing_extensions import Literal

import httpx

from ...types import (
    evaluation_dataset_list_params,
    evaluation_dataset_create_params,
    evaluation_dataset_update_params,
    evaluation_dataset_publish_params,
    evaluation_dataset_approve_batch_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from .test_cases import (
    TestCasesResource,
    AsyncTestCasesResource,
    TestCasesResourceWithRawResponse,
    AsyncTestCasesResourceWithRawResponse,
    TestCasesResourceWithStreamingResponse,
    AsyncTestCasesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from .generation_jobs import (
    GenerationJobsResource,
    AsyncGenerationJobsResource,
    GenerationJobsResourceWithRawResponse,
    AsyncGenerationJobsResourceWithRawResponse,
    GenerationJobsResourceWithStreamingResponse,
    AsyncGenerationJobsResourceWithStreamingResponse,
)
from .test_cases.test_cases import TestCasesResource, AsyncTestCasesResource
from ...types.evaluation_dataset import EvaluationDataset
from .evaluation_dataset_versions import (
    EvaluationDatasetVersionsResource,
    AsyncEvaluationDatasetVersionsResource,
    EvaluationDatasetVersionsResourceWithRawResponse,
    AsyncEvaluationDatasetVersionsResourceWithRawResponse,
    EvaluationDatasetVersionsResourceWithStreamingResponse,
    AsyncEvaluationDatasetVersionsResourceWithStreamingResponse,
)
from .autogenerated_draft_test_cases import (
    AutogeneratedDraftTestCasesResource,
    AsyncAutogeneratedDraftTestCasesResource,
    AutogeneratedDraftTestCasesResourceWithRawResponse,
    AsyncAutogeneratedDraftTestCasesResourceWithRawResponse,
    AutogeneratedDraftTestCasesResourceWithStreamingResponse,
    AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse,
)
from ...types.shared.generic_delete_response import GenericDeleteResponse
from ...types.publish_evaluation_dataset_draft_response import PublishEvaluationDatasetDraftResponse
from ...types.auto_generated_draft_test_case_approve_batch_response import (
    AutoGeneratedDraftTestCaseApproveBatchResponse,
)

__all__ = ["EvaluationDatasetsResource", "AsyncEvaluationDatasetsResource"]


class EvaluationDatasetsResource(SyncAPIResource):
    @cached_property
    def evaluation_dataset_versions(self) -> EvaluationDatasetVersionsResource:
        return EvaluationDatasetVersionsResource(self._client)

    @cached_property
    def test_cases(self) -> TestCasesResource:
        return TestCasesResource(self._client)

    @cached_property
    def autogenerated_draft_test_cases(self) -> AutogeneratedDraftTestCasesResource:
        return AutogeneratedDraftTestCasesResource(self._client)

    @cached_property
    def generation_jobs(self) -> GenerationJobsResource:
        return GenerationJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EvaluationDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return EvaluationDatasetsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        name: str,
        schema_type: Literal["GENERATION", "FLEXIBLE"],
        type: Literal["manual"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Creates a evaluation dataset

        ### Details

        This API can be used to create a evaluation dataset. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        dataset.

        Args:
          account_id: The ID of the account that owns the given entity.

          name: The name of the dataset

          schema_type: An enumeration.

          type: Manually or automatically generated dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        knowledge_base_id: str,
        name: str,
        schema_type: Literal["GENERATION"],
        type: Literal["autogenerated"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Creates a evaluation dataset

        ### Details

        This API can be used to create a evaluation dataset. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        dataset.

        Args:
          account_id: The ID of the account that owns the given entity.

          knowledge_base_id: ID of the knowledge base to generate the evaluation dataset from.

          name: The name of the dataset

          schema_type: The schema type of the dataset. Only GENERATION is currently supported.

          type: Manually or automatically generated dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        advanced_config: Dict[str, List[str]],
        harms_list: List[str],
        name: str,
        schema_type: Literal["GENERATION"],
        type: Literal["safety"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Creates a evaluation dataset

        ### Details

        This API can be used to create a evaluation dataset. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        dataset.

        Args:
          account_id: The ID of the account that owns the given entity.

          advanced_config: Advanced configuration for the safety evaluation dataset.

          harms_list: List of harms to generate the dataset for.

          name: The name of the dataset

          schema_type: The schema type of the dataset. Only GENERATION is currently supported.

          type: Manually or automatically generated dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "name", "schema_type"],
        ["account_id", "knowledge_base_id", "name", "schema_type"],
        ["account_id", "advanced_config", "harms_list", "name", "schema_type"],
    )
    def create(
        self,
        *,
        account_id: str,
        name: str,
        schema_type: Literal["GENERATION", "FLEXIBLE"] | Literal["GENERATION"],
        type: Literal["manual"] | Literal["autogenerated"] | Literal["safety"] | NotGiven = NOT_GIVEN,
        knowledge_base_id: str | NotGiven = NOT_GIVEN,
        advanced_config: Dict[str, List[str]] | NotGiven = NOT_GIVEN,
        harms_list: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        return self._post(
            "/v4/evaluation-datasets",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "schema_type": schema_type,
                    "type": type,
                    "knowledge_base_id": knowledge_base_id,
                    "advanced_config": advanced_config,
                    "harms_list": harms_list,
                },
                evaluation_dataset_create_params.EvaluationDatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDataset,
        )

    def retrieve(
        self,
        evaluation_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Gets the details of a dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDataset,
        )

    @overload
    def update(
        self,
        evaluation_dataset_id: str,
        *,
        name: str,
        restore: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Updates a evaluation dataset

        ### Details

        This API can be used to update the evaluation dataset that matches the ID that
        was passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation Dataset API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          name: The name of the dataset

          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        evaluation_dataset_id: str,
        *,
        restore: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Updates a evaluation dataset

        ### Details

        This API can be used to update the evaluation dataset that matches the ID that
        was passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation Dataset API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["name"], ["restore"])
    def update(
        self,
        evaluation_dataset_id: str,
        *,
        name: str | NotGiven = NOT_GIVEN,
        restore: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._patch(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}",
            body=maybe_transform(
                {
                    "name": name,
                    "restore": restore,
                },
                evaluation_dataset_update_params.EvaluationDatasetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDataset,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[EvaluationDataset]:
        """
        ### Description

        Lists all evaluation datasets accessible to the user.

        ### Details

        This API can be used to list evaluation datasets. If a user has access to
        multiple accounts, all evaluation datasets from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/evaluation-datasets",
            page=SyncPageResponse[EvaluationDataset],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    evaluation_dataset_list_params.EvaluationDatasetListParams,
                ),
            ),
            model=EvaluationDataset,
        )

    def approve_batch(
        self,
        evaluation_dataset_id: str,
        *,
        autogenerated_draft_test_cases: List[str],
        force: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCaseApproveBatchResponse:
        """
        Approve Auto Generated Test Cases Batch

        Args:
          autogenerated_draft_test_cases: Ids of auto generated draft test cases to be approved.

          force: Force approve a batch of autogenerated test case IDs for the evaluation dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/approve-batch",
            body=maybe_transform(
                {"autogenerated_draft_test_cases": autogenerated_draft_test_cases},
                evaluation_dataset_approve_batch_params.EvaluationDatasetApproveBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"force": force}, evaluation_dataset_approve_batch_params.EvaluationDatasetApproveBatchParams
                ),
            ),
            cast_to=AutoGeneratedDraftTestCaseApproveBatchResponse,
        )

    def publish(
        self,
        evaluation_dataset_id: str,
        *,
        force: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PublishEvaluationDatasetDraftResponse:
        """
        Publish Evaluation Dataset

        Args:
          force: Force approve an evaluation dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/publish",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"force": force}, evaluation_dataset_publish_params.EvaluationDatasetPublishParams
                ),
            ),
            cast_to=PublishEvaluationDatasetDraftResponse,
        )

    def remove(
        self,
        evaluation_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes the dataset, and all other entities associated with the dataset, such as
        test cases, evaluations and results.

        ### Details

        **This is a permanent and destructive action that cannot be undone.**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._delete(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncEvaluationDatasetsResource(AsyncAPIResource):
    @cached_property
    def evaluation_dataset_versions(self) -> AsyncEvaluationDatasetVersionsResource:
        return AsyncEvaluationDatasetVersionsResource(self._client)

    @cached_property
    def test_cases(self) -> AsyncTestCasesResource:
        return AsyncTestCasesResource(self._client)

    @cached_property
    def autogenerated_draft_test_cases(self) -> AsyncAutogeneratedDraftTestCasesResource:
        return AsyncAutogeneratedDraftTestCasesResource(self._client)

    @cached_property
    def generation_jobs(self) -> AsyncGenerationJobsResource:
        return AsyncGenerationJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEvaluationDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncEvaluationDatasetsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        name: str,
        schema_type: Literal["GENERATION", "FLEXIBLE"],
        type: Literal["manual"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Creates a evaluation dataset

        ### Details

        This API can be used to create a evaluation dataset. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        dataset.

        Args:
          account_id: The ID of the account that owns the given entity.

          name: The name of the dataset

          schema_type: An enumeration.

          type: Manually or automatically generated dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        knowledge_base_id: str,
        name: str,
        schema_type: Literal["GENERATION"],
        type: Literal["autogenerated"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Creates a evaluation dataset

        ### Details

        This API can be used to create a evaluation dataset. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        dataset.

        Args:
          account_id: The ID of the account that owns the given entity.

          knowledge_base_id: ID of the knowledge base to generate the evaluation dataset from.

          name: The name of the dataset

          schema_type: The schema type of the dataset. Only GENERATION is currently supported.

          type: Manually or automatically generated dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        advanced_config: Dict[str, List[str]],
        harms_list: List[str],
        name: str,
        schema_type: Literal["GENERATION"],
        type: Literal["safety"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Creates a evaluation dataset

        ### Details

        This API can be used to create a evaluation dataset. To use this API, review the
        request schema and pass in all fields that are required to create a evaluation
        dataset.

        Args:
          account_id: The ID of the account that owns the given entity.

          advanced_config: Advanced configuration for the safety evaluation dataset.

          harms_list: List of harms to generate the dataset for.

          name: The name of the dataset

          schema_type: The schema type of the dataset. Only GENERATION is currently supported.

          type: Manually or automatically generated dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "name", "schema_type"],
        ["account_id", "knowledge_base_id", "name", "schema_type"],
        ["account_id", "advanced_config", "harms_list", "name", "schema_type"],
    )
    async def create(
        self,
        *,
        account_id: str,
        name: str,
        schema_type: Literal["GENERATION", "FLEXIBLE"] | Literal["GENERATION"],
        type: Literal["manual"] | Literal["autogenerated"] | Literal["safety"] | NotGiven = NOT_GIVEN,
        knowledge_base_id: str | NotGiven = NOT_GIVEN,
        advanced_config: Dict[str, List[str]] | NotGiven = NOT_GIVEN,
        harms_list: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        return await self._post(
            "/v4/evaluation-datasets",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "schema_type": schema_type,
                    "type": type,
                    "knowledge_base_id": knowledge_base_id,
                    "advanced_config": advanced_config,
                    "harms_list": harms_list,
                },
                evaluation_dataset_create_params.EvaluationDatasetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDataset,
        )

    async def retrieve(
        self,
        evaluation_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Gets the details of a dataset.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDataset,
        )

    @overload
    async def update(
        self,
        evaluation_dataset_id: str,
        *,
        name: str,
        restore: Literal[False] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Updates a evaluation dataset

        ### Details

        This API can be used to update the evaluation dataset that matches the ID that
        was passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation Dataset API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          name: The name of the dataset

          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        evaluation_dataset_id: str,
        *,
        restore: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        """
        ### Description

        Updates a evaluation dataset

        ### Details

        This API can be used to update the evaluation dataset that matches the ID that
        was passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation Dataset API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["name"], ["restore"])
    async def update(
        self,
        evaluation_dataset_id: str,
        *,
        name: str | NotGiven = NOT_GIVEN,
        restore: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDataset:
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._patch(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "restore": restore,
                },
                evaluation_dataset_update_params.EvaluationDatasetUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDataset,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[EvaluationDataset, AsyncPageResponse[EvaluationDataset]]:
        """
        ### Description

        Lists all evaluation datasets accessible to the user.

        ### Details

        This API can be used to list evaluation datasets. If a user has access to
        multiple accounts, all evaluation datasets from all accounts the user is
        associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/evaluation-datasets",
            page=AsyncPageResponse[EvaluationDataset],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    evaluation_dataset_list_params.EvaluationDatasetListParams,
                ),
            ),
            model=EvaluationDataset,
        )

    async def approve_batch(
        self,
        evaluation_dataset_id: str,
        *,
        autogenerated_draft_test_cases: List[str],
        force: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AutoGeneratedDraftTestCaseApproveBatchResponse:
        """
        Approve Auto Generated Test Cases Batch

        Args:
          autogenerated_draft_test_cases: Ids of auto generated draft test cases to be approved.

          force: Force approve a batch of autogenerated test case IDs for the evaluation dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/approve-batch",
            body=await async_maybe_transform(
                {"autogenerated_draft_test_cases": autogenerated_draft_test_cases},
                evaluation_dataset_approve_batch_params.EvaluationDatasetApproveBatchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"force": force}, evaluation_dataset_approve_batch_params.EvaluationDatasetApproveBatchParams
                ),
            ),
            cast_to=AutoGeneratedDraftTestCaseApproveBatchResponse,
        )

    async def publish(
        self,
        evaluation_dataset_id: str,
        *,
        force: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PublishEvaluationDatasetDraftResponse:
        """
        Publish Evaluation Dataset

        Args:
          force: Force approve an evaluation dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/publish",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"force": force}, evaluation_dataset_publish_params.EvaluationDatasetPublishParams
                ),
            ),
            cast_to=PublishEvaluationDatasetDraftResponse,
        )

    async def remove(
        self,
        evaluation_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes the dataset, and all other entities associated with the dataset, such as
        test cases, evaluations and results.

        ### Details

        **This is a permanent and destructive action that cannot be undone.**

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._delete(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class EvaluationDatasetsResourceWithRawResponse:
    def __init__(self, evaluation_datasets: EvaluationDatasetsResource) -> None:
        self._evaluation_datasets = evaluation_datasets

        self.create = to_raw_response_wrapper(
            evaluation_datasets.create,
        )
        self.retrieve = to_raw_response_wrapper(
            evaluation_datasets.retrieve,
        )
        self.update = to_raw_response_wrapper(
            evaluation_datasets.update,
        )
        self.list = to_raw_response_wrapper(
            evaluation_datasets.list,
        )
        self.approve_batch = to_raw_response_wrapper(
            evaluation_datasets.approve_batch,
        )
        self.publish = to_raw_response_wrapper(
            evaluation_datasets.publish,
        )
        self.remove = to_raw_response_wrapper(
            evaluation_datasets.remove,
        )

    @cached_property
    def evaluation_dataset_versions(self) -> EvaluationDatasetVersionsResourceWithRawResponse:
        return EvaluationDatasetVersionsResourceWithRawResponse(self._evaluation_datasets.evaluation_dataset_versions)

    @cached_property
    def test_cases(self) -> TestCasesResourceWithRawResponse:
        return TestCasesResourceWithRawResponse(self._evaluation_datasets.test_cases)

    @cached_property
    def autogenerated_draft_test_cases(self) -> AutogeneratedDraftTestCasesResourceWithRawResponse:
        return AutogeneratedDraftTestCasesResourceWithRawResponse(
            self._evaluation_datasets.autogenerated_draft_test_cases
        )

    @cached_property
    def generation_jobs(self) -> GenerationJobsResourceWithRawResponse:
        return GenerationJobsResourceWithRawResponse(self._evaluation_datasets.generation_jobs)


class AsyncEvaluationDatasetsResourceWithRawResponse:
    def __init__(self, evaluation_datasets: AsyncEvaluationDatasetsResource) -> None:
        self._evaluation_datasets = evaluation_datasets

        self.create = async_to_raw_response_wrapper(
            evaluation_datasets.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            evaluation_datasets.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            evaluation_datasets.update,
        )
        self.list = async_to_raw_response_wrapper(
            evaluation_datasets.list,
        )
        self.approve_batch = async_to_raw_response_wrapper(
            evaluation_datasets.approve_batch,
        )
        self.publish = async_to_raw_response_wrapper(
            evaluation_datasets.publish,
        )
        self.remove = async_to_raw_response_wrapper(
            evaluation_datasets.remove,
        )

    @cached_property
    def evaluation_dataset_versions(self) -> AsyncEvaluationDatasetVersionsResourceWithRawResponse:
        return AsyncEvaluationDatasetVersionsResourceWithRawResponse(
            self._evaluation_datasets.evaluation_dataset_versions
        )

    @cached_property
    def test_cases(self) -> AsyncTestCasesResourceWithRawResponse:
        return AsyncTestCasesResourceWithRawResponse(self._evaluation_datasets.test_cases)

    @cached_property
    def autogenerated_draft_test_cases(self) -> AsyncAutogeneratedDraftTestCasesResourceWithRawResponse:
        return AsyncAutogeneratedDraftTestCasesResourceWithRawResponse(
            self._evaluation_datasets.autogenerated_draft_test_cases
        )

    @cached_property
    def generation_jobs(self) -> AsyncGenerationJobsResourceWithRawResponse:
        return AsyncGenerationJobsResourceWithRawResponse(self._evaluation_datasets.generation_jobs)


class EvaluationDatasetsResourceWithStreamingResponse:
    def __init__(self, evaluation_datasets: EvaluationDatasetsResource) -> None:
        self._evaluation_datasets = evaluation_datasets

        self.create = to_streamed_response_wrapper(
            evaluation_datasets.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            evaluation_datasets.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            evaluation_datasets.update,
        )
        self.list = to_streamed_response_wrapper(
            evaluation_datasets.list,
        )
        self.approve_batch = to_streamed_response_wrapper(
            evaluation_datasets.approve_batch,
        )
        self.publish = to_streamed_response_wrapper(
            evaluation_datasets.publish,
        )
        self.remove = to_streamed_response_wrapper(
            evaluation_datasets.remove,
        )

    @cached_property
    def evaluation_dataset_versions(self) -> EvaluationDatasetVersionsResourceWithStreamingResponse:
        return EvaluationDatasetVersionsResourceWithStreamingResponse(
            self._evaluation_datasets.evaluation_dataset_versions
        )

    @cached_property
    def test_cases(self) -> TestCasesResourceWithStreamingResponse:
        return TestCasesResourceWithStreamingResponse(self._evaluation_datasets.test_cases)

    @cached_property
    def autogenerated_draft_test_cases(self) -> AutogeneratedDraftTestCasesResourceWithStreamingResponse:
        return AutogeneratedDraftTestCasesResourceWithStreamingResponse(
            self._evaluation_datasets.autogenerated_draft_test_cases
        )

    @cached_property
    def generation_jobs(self) -> GenerationJobsResourceWithStreamingResponse:
        return GenerationJobsResourceWithStreamingResponse(self._evaluation_datasets.generation_jobs)


class AsyncEvaluationDatasetsResourceWithStreamingResponse:
    def __init__(self, evaluation_datasets: AsyncEvaluationDatasetsResource) -> None:
        self._evaluation_datasets = evaluation_datasets

        self.create = async_to_streamed_response_wrapper(
            evaluation_datasets.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            evaluation_datasets.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            evaluation_datasets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            evaluation_datasets.list,
        )
        self.approve_batch = async_to_streamed_response_wrapper(
            evaluation_datasets.approve_batch,
        )
        self.publish = async_to_streamed_response_wrapper(
            evaluation_datasets.publish,
        )
        self.remove = async_to_streamed_response_wrapper(
            evaluation_datasets.remove,
        )

    @cached_property
    def evaluation_dataset_versions(self) -> AsyncEvaluationDatasetVersionsResourceWithStreamingResponse:
        return AsyncEvaluationDatasetVersionsResourceWithStreamingResponse(
            self._evaluation_datasets.evaluation_dataset_versions
        )

    @cached_property
    def test_cases(self) -> AsyncTestCasesResourceWithStreamingResponse:
        return AsyncTestCasesResourceWithStreamingResponse(self._evaluation_datasets.test_cases)

    @cached_property
    def autogenerated_draft_test_cases(self) -> AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse:
        return AsyncAutogeneratedDraftTestCasesResourceWithStreamingResponse(
            self._evaluation_datasets.autogenerated_draft_test_cases
        )

    @cached_property
    def generation_jobs(self) -> AsyncGenerationJobsResourceWithStreamingResponse:
        return AsyncGenerationJobsResourceWithStreamingResponse(self._evaluation_datasets.generation_jobs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .feedback import (
    FeedbackResource,
    AsyncFeedbackResource,
    FeedbackResourceWithRawResponse,
    AsyncFeedbackResourceWithRawResponse,
    FeedbackResourceWithStreamingResponse,
    AsyncFeedbackResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.chat_threads import message_update_params
from ....types.applications.chat_thread_history import ChatThreadHistory
from ....types.chat_threads.message_update_response import MessageUpdateResponse

__all__ = ["MessagesResource", "AsyncMessagesResource"]


class MessagesResource(SyncAPIResource):
    @cached_property
    def feedback(self) -> FeedbackResource:
        return FeedbackResource(self._client)

    @cached_property
    def with_raw_response(self) -> MessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return MessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return MessagesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        thread_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatThreadHistory:
        """
        Get Thread Messages

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return self._get(
            f"/v4/threads/{thread_id}/messages",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatThreadHistory,
        )

    def update(
        self,
        application_interaction_id: str,
        *,
        thread_id: str,
        output_text: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MessageUpdateResponse:
        """
        Patch Chat Thread Entry

        Args:
          output_text: The output text generated by the application.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        if not application_interaction_id:
            raise ValueError(
                f"Expected a non-empty value for `application_interaction_id` but received {application_interaction_id!r}"
            )
        return self._patch(
            f"/v4/threads/{thread_id}/messages/{application_interaction_id}",
            body=maybe_transform({"output_text": output_text}, message_update_params.MessageUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageUpdateResponse,
        )


class AsyncMessagesResource(AsyncAPIResource):
    @cached_property
    def feedback(self) -> AsyncFeedbackResource:
        return AsyncFeedbackResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncMessagesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        thread_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatThreadHistory:
        """
        Get Thread Messages

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        return await self._get(
            f"/v4/threads/{thread_id}/messages",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatThreadHistory,
        )

    async def update(
        self,
        application_interaction_id: str,
        *,
        thread_id: str,
        output_text: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MessageUpdateResponse:
        """
        Patch Chat Thread Entry

        Args:
          output_text: The output text generated by the application.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not thread_id:
            raise ValueError(f"Expected a non-empty value for `thread_id` but received {thread_id!r}")
        if not application_interaction_id:
            raise ValueError(
                f"Expected a non-empty value for `application_interaction_id` but received {application_interaction_id!r}"
            )
        return await self._patch(
            f"/v4/threads/{thread_id}/messages/{application_interaction_id}",
            body=await async_maybe_transform({"output_text": output_text}, message_update_params.MessageUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageUpdateResponse,
        )


class MessagesResourceWithRawResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.retrieve = to_raw_response_wrapper(
            messages.retrieve,
        )
        self.update = to_raw_response_wrapper(
            messages.update,
        )

    @cached_property
    def feedback(self) -> FeedbackResourceWithRawResponse:
        return FeedbackResourceWithRawResponse(self._messages.feedback)


class AsyncMessagesResourceWithRawResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.retrieve = async_to_raw_response_wrapper(
            messages.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            messages.update,
        )

    @cached_property
    def feedback(self) -> AsyncFeedbackResourceWithRawResponse:
        return AsyncFeedbackResourceWithRawResponse(self._messages.feedback)


class MessagesResourceWithStreamingResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.retrieve = to_streamed_response_wrapper(
            messages.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            messages.update,
        )

    @cached_property
    def feedback(self) -> FeedbackResourceWithStreamingResponse:
        return FeedbackResourceWithStreamingResponse(self._messages.feedback)


class AsyncMessagesResourceWithStreamingResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.retrieve = async_to_streamed_response_wrapper(
            messages.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            messages.update,
        )

    @cached_property
    def feedback(self) -> AsyncFeedbackResourceWithStreamingResponse:
        return AsyncFeedbackResourceWithStreamingResponse(self._messages.feedback)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .model_deployment import ModelDeployment as ModelDeployment
from .deployment_list_params import DeploymentListParams as DeploymentListParams
from .embedding_create_params import EmbeddingCreateParams as EmbeddingCreateParams
from .reranking_create_params import RerankingCreateParams as RerankingCreateParams
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .deployment_create_params import DeploymentCreateParams as DeploymentCreateParams
from .deployment_update_params import DeploymentUpdateParams as DeploymentUpdateParams
from .deployment_execute_params import DeploymentExecuteParams as DeploymentExecuteParams
from .deployment_list_all_params import DeploymentListAllParams as DeploymentListAllParams
from .paginated_model_deployments import PaginatedModelDeployments as PaginatedModelDeployments
from .chat_completion_create_params import ChatCompletionCreateParams as ChatCompletionCreateParams
from .usage_statistic_retrieve_params import UsageStatisticRetrieveParams as UsageStatisticRetrieveParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "ChatCompletionsResponse",
    "ChatCompletion",
    "ChatCompletionMessage",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsAssistantMessage",
    "ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage",
    "TokenUsage",
]


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts(
    BaseModel
):
    text: str

    type: Optional[Literal["text"]] = None


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL(
    BaseModel
):
    url: str
    """The URL of the image. Note: only OpenAI supports this."""

    detail: Optional[Literal["low", "high", "auto"]] = None
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts(
    BaseModel
):
    image_url: ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentPartsImageURL
    """Specifies the image URL and level of detail. Only supported by OpenAI models"""

    type: Optional[Literal["image_url"]] = None


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData(
    BaseModel
):
    data: str
    """The base64-encoded image data."""

    media_type: str
    """The media/mime type of the image data.

    For example, 'image/png'. Check providers' documentation for supported media
    types.
    """

    detail: Optional[Literal["low", "high", "auto"]] = None
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""

    type: Optional[Literal["base64"]] = None
    """The type of the image data. Only base64 is supported."""


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts(
    BaseModel
):
    image_data: ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentPartsImageData
    """Specifies inline image data"""

    type: Optional[Literal["image_data"]] = None


ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1: TypeAlias = Annotated[
    Union[
        ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsTextUserMessageContentParts,
        ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageURLUserMessageContentParts,
        ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1EgpAPIBackendServerAPIModelsEgpModelsImageDataUserMessageContentParts,
    ],
    PropertyInfo(discriminator="type"),
]


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage(BaseModel):
    content: Union[str, List[ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessageContentUnionMember1]]
    """Input from the user.

    Can either be text or a list of content parts. Not all models support image
    content parts, or multiple parts.
    """

    role: Optional[Literal["user"]] = None
    """The role of the message. Must be set to 'user'.

    A user message is a message from the user to the AI. This should be the message
    used to send end user input to the AI.
    """


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsAssistantMessage(BaseModel):
    content: str
    """Text response from the assistant"""

    role: Optional[Literal["assistant"]] = None
    """The role of the message. Must be set to 'assistant'.

    An assistant message is a message from the AI to the client. It is different
    from an agent message in that it cannot contain a tool request. It is simply a
    direct response from the AI to the client.
    """


class ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage(BaseModel):
    content: str
    """Text input from the system."""

    role: Optional[Literal["system"]] = None
    """The role of the message. Must be set to 'system'.

    A system message is different from other messages in that it does not originate
    from a party engaged in a user/AI conversation. Instead, it is a message that is
    injected by either the application or system to guide the conversation. For
    example, a system message may be used as initial instructions for an AI entity
    or to tell the AI that it did not do something correctly.
    """


ChatCompletionMessage: TypeAlias = Annotated[
    Union[
        ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsUserMessage,
        ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsAssistantMessage,
        ChatCompletionMessageEgpAPIBackendServerAPIModelsEgpModelsSystemMessage,
    ],
    PropertyInfo(discriminator="role"),
]


class ChatCompletion(BaseModel):
    message: ChatCompletionMessage

    finish_reason: Optional[str] = None


class TokenUsage(BaseModel):
    total: int
    """Total number of tokens in both the prompt and the completion."""

    completion: Optional[int] = None
    """Number of tokens in the completion."""

    prompt: Optional[int] = None
    """Number of tokens in the prompt."""


class ChatCompletionsResponse(BaseModel):
    chat_completion: ChatCompletion

    token_usage: Optional[TokenUsage] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ApplicationVariantProcessParams", "History", "Overrides"]


class ApplicationVariantProcessParams(TypedDict, total=False):
    inputs: Required[object]
    """Input data for the application. You must provide inputs for each input node"""

    history: Iterable[History]
    """History of the application"""

    overrides: Dict[str, Overrides]
    """Optional overrides for the application"""

    stream: bool
    """Control to have streaming of the endpoint.

    If the last node before the output is a completion node, you can set this to
    true to get the output as soon as the completion node has a token
    """


class History(TypedDict, total=False):
    request: Required[str]
    """Request inputs"""

    response: Required[str]
    """Response outputs"""

    session_data: object
    """Session data corresponding to the request response pair"""


class Overrides(TypedDict, total=False):
    artifact_ids_filter: List[str]

    artifact_name_regex: List[str]

    type: Literal["knowledge_base_schema"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ParameterSchemaFieldParam"]


class ParameterSchemaFieldParam(TypedDict, total=False):
    description: Required[str]

    name: Required[str]

    required: Required[bool]

    type: Required[Literal["str", "int", "float", "bool"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["MultiturnAnnotationConfigParam"]


class MultiturnAnnotationConfigParam(TypedDict, total=False):
    messages_loc: Required[List[str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import Literal

import httpx

from ...types import model_list_params, model_create_params, model_update_params, model_retrieve_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .rerankings import (
    RerankingsResource,
    AsyncRerankingsResource,
    RerankingsResourceWithRawResponse,
    AsyncRerankingsResourceWithRawResponse,
    RerankingsResourceWithStreamingResponse,
    AsyncRerankingsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from .chat_completions import (
    ChatCompletionsResource,
    AsyncChatCompletionsResource,
    ChatCompletionsResourceWithRawResponse,
    AsyncChatCompletionsResourceWithRawResponse,
    ChatCompletionsResourceWithStreamingResponse,
    AsyncChatCompletionsResourceWithStreamingResponse,
)
from .usage_statistics import (
    UsageStatisticsResource,
    AsyncUsageStatisticsResource,
    UsageStatisticsResourceWithRawResponse,
    AsyncUsageStatisticsResourceWithRawResponse,
    UsageStatisticsResourceWithStreamingResponse,
    AsyncUsageStatisticsResourceWithStreamingResponse,
)
from ...types.model_instance import ModelInstance
from .deployments.deployments import DeploymentsResource, AsyncDeploymentsResource
from ...types.model_instance_with_views import ModelInstanceWithViews
from ...types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["ModelsResource", "AsyncModelsResource"]


class ModelsResource(SyncAPIResource):
    @cached_property
    def deployments(self) -> DeploymentsResource:
        return DeploymentsResource(self._client)

    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        return EmbeddingsResource(self._client)

    @cached_property
    def rerankings(self) -> RerankingsResource:
        return RerankingsResource(self._client)

    @cached_property
    def completions(self) -> CompletionsResource:
        return CompletionsResource(self._client)

    @cached_property
    def chat_completions(self) -> ChatCompletionsResource:
        return ChatCompletionsResource(self._client)

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResource:
        return UsageStatisticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ModelsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"],
        name: str,
        base_model_id: str | NotGiven = NOT_GIVEN,
        base_model_metadata: model_create_params.BaseModelMetadata | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        model_card: str | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        model_group_id: str | NotGiven = NOT_GIVEN,
        model_template_id: str | NotGiven = NOT_GIVEN,
        model_vendor: Literal["OPENAI", "COHERE", "GOOGLE", "ANTHROPIC", "LAUNCH", "LLMENGINE", "BEDROCK", "OTHER"]
        | NotGiven = NOT_GIVEN,
        training_data_card: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelInstance:
        """
        ### Description

        Creates and hosts a model based on a model template.

        Base embedding models, chunk ranking functions, and LLMs are often not
        sufficient for customer use cases. We have shown in various blogs that
        fine-tuning these models on customer data can lead to significant improvements
        in performance.

        1. [We Fine-Tuned GPT-4 to Beat the Industry Standard for Text2SQL](https://scale.com/blog/text2sql-fine-tuning)
        2. [OpenAI Names Scale as Preferred Partner to Fine-Tune GPT-3.5](https://scale.com/blog/open-ai-scale-partnership-gpt-3-5-fine-tuning)
        3. [How to Fine-Tune GPT-3.5 Turbo With OpenAI API](https://scale.com/blog/fine-tune-gpt-3.5)

        ### Details

        Before creating a model, you must first create a model template. A model
        template serves 2 purposes. First, it provides common scaffolding that is static
        across multiple models. Second, it exposes several variables that can be
        injected at model creation time to customize the model.

        For example, a model template can define a docker image that contains code to
        run a HuggingFace or SentenceTransformers model. This docker image code also
        accepts environment variables that can be set to swap out the model weights or
        model name. Refer to the Create Model Template API for more details.

        To create a new model, users must refer to an existing model template and
        provide the necessary parameters the the model template requires in its
        `model_creation_parameters_schema` field. The combination of the model template
        and the model creation parameters will be used to create and deploy a new model.

        Once a model has been created, it can be executed by calling the Execute Model
        API.

        ### Coming Soon

        Some of our EGP APIs depend on models, for example Knowledge Base APIs depend on
        embedding models, Chunk Ranking APIs depend on ranking models, and Completion
        APIs depend on LLMs.

        In the near future, if a model is created from a model template that is
        compatible with one of these APIs (based on the model template's
        `model_type field`), the model will automatically be registered with the API.
        This will allow users to immediately start using the model with those API
        without any additional setup.

        Args:
          account_id: The ID of the account that owns the given entity.

          model_type: An enum representing the different types of models supported.

              Attributes: COMPLETION: Denotes that the model type is completion.
              CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
              that the model type is agent. EMBEDDING: Denotes that the model type is
              embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
              that the model type is generic.

          model_vendor: An enum representing the different types of model vendors supported.

              Attributes: OPENAI: Denotes that the model vendor is OpenAI. COHERE: Denotes
              that the model vendor is Cohere. GOOGLE: Denotes that the model vendor is
              Google. ANTHROPIC: Denotes that the model vendor is Anthropic. LLMENGINE:
              Denotes that the model vendor is LLM Engine. OTHER: Denotes that the model
              vendor is Other.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/models",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "model_type": model_type,
                    "name": name,
                    "base_model_id": base_model_id,
                    "base_model_metadata": base_model_metadata,
                    "description": description,
                    "model_card": model_card,
                    "model_creation_parameters": model_creation_parameters,
                    "model_group_id": model_group_id,
                    "model_template_id": model_template_id,
                    "model_vendor": model_vendor,
                    "training_data_card": training_data_card,
                },
                model_create_params.ModelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelInstance,
        )

    def retrieve(
        self,
        model_id: str,
        *,
        view: List[Literal["Deployments", "ModelGroup"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelInstanceWithViews:
        """
        ### Description

        Gets the details of a model

        ### Details

        This API can be used to get information about a single model by ID. To use this
        API, pass in the `id` that was returned from your Create Model API call as a
        path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._get(
            f"/v4/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"view": view}, model_retrieve_params.ModelRetrieveParams),
            ),
            cast_to=ModelInstanceWithViews,
        )

    def update(
        self,
        model_id: str,
        *,
        base_model_id: str | NotGiven = NOT_GIVEN,
        base_model_metadata: model_update_params.BaseModelMetadata | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        model_card: str | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        model_group_id: str | NotGiven = NOT_GIVEN,
        model_template_id: str | NotGiven = NOT_GIVEN,
        model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"]
        | NotGiven = NOT_GIVEN,
        model_vendor: Literal["OPENAI", "COHERE", "GOOGLE", "ANTHROPIC", "LAUNCH", "LLMENGINE", "BEDROCK", "OTHER"]
        | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        training_data_card: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelInstance:
        """
        ### Description

        Updates a model

        ### Details

        This API can be used to update the model that matches the ID that was passed in
        as a path parameter. To use this API, pass in the `id` that was returned from
        your Create Model API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          model_type: An enum representing the different types of models supported.

              Attributes: COMPLETION: Denotes that the model type is completion.
              CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
              that the model type is agent. EMBEDDING: Denotes that the model type is
              embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
              that the model type is generic.

          model_vendor: An enum representing the different types of model vendors supported.

              Attributes: OPENAI: Denotes that the model vendor is OpenAI. COHERE: Denotes
              that the model vendor is Cohere. GOOGLE: Denotes that the model vendor is
              Google. ANTHROPIC: Denotes that the model vendor is Anthropic. LLMENGINE:
              Denotes that the model vendor is LLM Engine. OTHER: Denotes that the model
              vendor is Other.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._patch(
            f"/v4/models/{model_id}",
            body=maybe_transform(
                {
                    "base_model_id": base_model_id,
                    "base_model_metadata": base_model_metadata,
                    "description": description,
                    "model_card": model_card,
                    "model_creation_parameters": model_creation_parameters,
                    "model_group_id": model_group_id,
                    "model_template_id": model_template_id,
                    "model_type": model_type,
                    "model_vendor": model_vendor,
                    "name": name,
                    "training_data_card": training_data_card,
                },
                model_update_params.ModelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelInstance,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        model_group_id: Union[int, str] | NotGiven = NOT_GIVEN,
        model_type: Union[int, str] | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "model_vendor:asc",
                "model_vendor:desc",
                "base_model_id:asc",
                "base_model_id:desc",
                "base_model_metadata:asc",
                "base_model_metadata:desc",
                "model_creation_parameters:asc",
                "model_creation_parameters:desc",
                "model_card:asc",
                "model_card:desc",
                "training_data_card:asc",
                "training_data_card:desc",
                "description:asc",
                "description:desc",
                "model_template_id:asc",
                "model_template_id:desc",
                "model_group_id:asc",
                "model_group_id:desc",
                "model_group:asc",
                "model_group:desc",
                "request_schema:asc",
                "request_schema:desc",
                "response_schema:asc",
                "response_schema:desc",
                "deployment_count:asc",
                "deployment_count:desc",
                "supports_multi_turn:asc",
                "supports_multi_turn:desc",
                "deployments:asc",
                "deployments:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
                "model_type:asc",
                "model_type:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        view: List[Literal["Deployments", "ModelGroup"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ModelInstanceWithViews]:
        """
        ### Description

        Lists all models accessible to the user.

        ### Details

        This API can be used to list models. If a user has access to multiple accounts,
        all models from all accounts the user is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/models",
            page=SyncPageResponse[ModelInstanceWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "model_group_id": model_group_id,
                        "model_type": model_type,
                        "page": page,
                        "sort_by": sort_by,
                        "view": view,
                    },
                    model_list_params.ModelListParams,
                ),
            ),
            model=ModelInstanceWithViews,
        )

    def delete(
        self,
        model_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a model

        ### Details

        This API can be used to delete a model by ID. To use this API, pass in the `id`
        that was returned from your Create Model API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return self._delete(
            f"/v4/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncModelsResource(AsyncAPIResource):
    @cached_property
    def deployments(self) -> AsyncDeploymentsResource:
        return AsyncDeploymentsResource(self._client)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        return AsyncEmbeddingsResource(self._client)

    @cached_property
    def rerankings(self) -> AsyncRerankingsResource:
        return AsyncRerankingsResource(self._client)

    @cached_property
    def completions(self) -> AsyncCompletionsResource:
        return AsyncCompletionsResource(self._client)

    @cached_property
    def chat_completions(self) -> AsyncChatCompletionsResource:
        return AsyncChatCompletionsResource(self._client)

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResource:
        return AsyncUsageStatisticsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncModelsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"],
        name: str,
        base_model_id: str | NotGiven = NOT_GIVEN,
        base_model_metadata: model_create_params.BaseModelMetadata | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        model_card: str | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        model_group_id: str | NotGiven = NOT_GIVEN,
        model_template_id: str | NotGiven = NOT_GIVEN,
        model_vendor: Literal["OPENAI", "COHERE", "GOOGLE", "ANTHROPIC", "LAUNCH", "LLMENGINE", "BEDROCK", "OTHER"]
        | NotGiven = NOT_GIVEN,
        training_data_card: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelInstance:
        """
        ### Description

        Creates and hosts a model based on a model template.

        Base embedding models, chunk ranking functions, and LLMs are often not
        sufficient for customer use cases. We have shown in various blogs that
        fine-tuning these models on customer data can lead to significant improvements
        in performance.

        1. [We Fine-Tuned GPT-4 to Beat the Industry Standard for Text2SQL](https://scale.com/blog/text2sql-fine-tuning)
        2. [OpenAI Names Scale as Preferred Partner to Fine-Tune GPT-3.5](https://scale.com/blog/open-ai-scale-partnership-gpt-3-5-fine-tuning)
        3. [How to Fine-Tune GPT-3.5 Turbo With OpenAI API](https://scale.com/blog/fine-tune-gpt-3.5)

        ### Details

        Before creating a model, you must first create a model template. A model
        template serves 2 purposes. First, it provides common scaffolding that is static
        across multiple models. Second, it exposes several variables that can be
        injected at model creation time to customize the model.

        For example, a model template can define a docker image that contains code to
        run a HuggingFace or SentenceTransformers model. This docker image code also
        accepts environment variables that can be set to swap out the model weights or
        model name. Refer to the Create Model Template API for more details.

        To create a new model, users must refer to an existing model template and
        provide the necessary parameters the the model template requires in its
        `model_creation_parameters_schema` field. The combination of the model template
        and the model creation parameters will be used to create and deploy a new model.

        Once a model has been created, it can be executed by calling the Execute Model
        API.

        ### Coming Soon

        Some of our EGP APIs depend on models, for example Knowledge Base APIs depend on
        embedding models, Chunk Ranking APIs depend on ranking models, and Completion
        APIs depend on LLMs.

        In the near future, if a model is created from a model template that is
        compatible with one of these APIs (based on the model template's
        `model_type field`), the model will automatically be registered with the API.
        This will allow users to immediately start using the model with those API
        without any additional setup.

        Args:
          account_id: The ID of the account that owns the given entity.

          model_type: An enum representing the different types of models supported.

              Attributes: COMPLETION: Denotes that the model type is completion.
              CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
              that the model type is agent. EMBEDDING: Denotes that the model type is
              embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
              that the model type is generic.

          model_vendor: An enum representing the different types of model vendors supported.

              Attributes: OPENAI: Denotes that the model vendor is OpenAI. COHERE: Denotes
              that the model vendor is Cohere. GOOGLE: Denotes that the model vendor is
              Google. ANTHROPIC: Denotes that the model vendor is Anthropic. LLMENGINE:
              Denotes that the model vendor is LLM Engine. OTHER: Denotes that the model
              vendor is Other.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/models",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "model_type": model_type,
                    "name": name,
                    "base_model_id": base_model_id,
                    "base_model_metadata": base_model_metadata,
                    "description": description,
                    "model_card": model_card,
                    "model_creation_parameters": model_creation_parameters,
                    "model_group_id": model_group_id,
                    "model_template_id": model_template_id,
                    "model_vendor": model_vendor,
                    "training_data_card": training_data_card,
                },
                model_create_params.ModelCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelInstance,
        )

    async def retrieve(
        self,
        model_id: str,
        *,
        view: List[Literal["Deployments", "ModelGroup"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelInstanceWithViews:
        """
        ### Description

        Gets the details of a model

        ### Details

        This API can be used to get information about a single model by ID. To use this
        API, pass in the `id` that was returned from your Create Model API call as a
        path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._get(
            f"/v4/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"view": view}, model_retrieve_params.ModelRetrieveParams),
            ),
            cast_to=ModelInstanceWithViews,
        )

    async def update(
        self,
        model_id: str,
        *,
        base_model_id: str | NotGiven = NOT_GIVEN,
        base_model_metadata: model_update_params.BaseModelMetadata | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        model_card: str | NotGiven = NOT_GIVEN,
        model_creation_parameters: object | NotGiven = NOT_GIVEN,
        model_group_id: str | NotGiven = NOT_GIVEN,
        model_template_id: str | NotGiven = NOT_GIVEN,
        model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"]
        | NotGiven = NOT_GIVEN,
        model_vendor: Literal["OPENAI", "COHERE", "GOOGLE", "ANTHROPIC", "LAUNCH", "LLMENGINE", "BEDROCK", "OTHER"]
        | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        training_data_card: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelInstance:
        """
        ### Description

        Updates a model

        ### Details

        This API can be used to update the model that matches the ID that was passed in
        as a path parameter. To use this API, pass in the `id` that was returned from
        your Create Model API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          model_type: An enum representing the different types of models supported.

              Attributes: COMPLETION: Denotes that the model type is completion.
              CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
              that the model type is agent. EMBEDDING: Denotes that the model type is
              embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
              that the model type is generic.

          model_vendor: An enum representing the different types of model vendors supported.

              Attributes: OPENAI: Denotes that the model vendor is OpenAI. COHERE: Denotes
              that the model vendor is Cohere. GOOGLE: Denotes that the model vendor is
              Google. ANTHROPIC: Denotes that the model vendor is Anthropic. LLMENGINE:
              Denotes that the model vendor is LLM Engine. OTHER: Denotes that the model
              vendor is Other.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._patch(
            f"/v4/models/{model_id}",
            body=await async_maybe_transform(
                {
                    "base_model_id": base_model_id,
                    "base_model_metadata": base_model_metadata,
                    "description": description,
                    "model_card": model_card,
                    "model_creation_parameters": model_creation_parameters,
                    "model_group_id": model_group_id,
                    "model_template_id": model_template_id,
                    "model_type": model_type,
                    "model_vendor": model_vendor,
                    "name": name,
                    "training_data_card": training_data_card,
                },
                model_update_params.ModelUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelInstance,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        model_group_id: Union[int, str] | NotGiven = NOT_GIVEN,
        model_type: Union[int, str] | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "model_vendor:asc",
                "model_vendor:desc",
                "base_model_id:asc",
                "base_model_id:desc",
                "base_model_metadata:asc",
                "base_model_metadata:desc",
                "model_creation_parameters:asc",
                "model_creation_parameters:desc",
                "model_card:asc",
                "model_card:desc",
                "training_data_card:asc",
                "training_data_card:desc",
                "description:asc",
                "description:desc",
                "model_template_id:asc",
                "model_template_id:desc",
                "model_group_id:asc",
                "model_group_id:desc",
                "model_group:asc",
                "model_group:desc",
                "request_schema:asc",
                "request_schema:desc",
                "response_schema:asc",
                "response_schema:desc",
                "deployment_count:asc",
                "deployment_count:desc",
                "supports_multi_turn:asc",
                "supports_multi_turn:desc",
                "deployments:asc",
                "deployments:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
                "model_type:asc",
                "model_type:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        view: List[Literal["Deployments", "ModelGroup"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ModelInstanceWithViews, AsyncPageResponse[ModelInstanceWithViews]]:
        """
        ### Description

        Lists all models accessible to the user.

        ### Details

        This API can be used to list models. If a user has access to multiple accounts,
        all models from all accounts the user is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/models",
            page=AsyncPageResponse[ModelInstanceWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "model_group_id": model_group_id,
                        "model_type": model_type,
                        "page": page,
                        "sort_by": sort_by,
                        "view": view,
                    },
                    model_list_params.ModelListParams,
                ),
            ),
            model=ModelInstanceWithViews,
        )

    async def delete(
        self,
        model_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a model

        ### Details

        This API can be used to delete a model by ID. To use this API, pass in the `id`
        that was returned from your Create Model API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_id:
            raise ValueError(f"Expected a non-empty value for `model_id` but received {model_id!r}")
        return await self._delete(
            f"/v4/models/{model_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class ModelsResourceWithRawResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.create = to_raw_response_wrapper(
            models.create,
        )
        self.retrieve = to_raw_response_wrapper(
            models.retrieve,
        )
        self.update = to_raw_response_wrapper(
            models.update,
        )
        self.list = to_raw_response_wrapper(
            models.list,
        )
        self.delete = to_raw_response_wrapper(
            models.delete,
        )

    @cached_property
    def deployments(self) -> DeploymentsResourceWithRawResponse:
        return DeploymentsResourceWithRawResponse(self._models.deployments)

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithRawResponse:
        return EmbeddingsResourceWithRawResponse(self._models.embeddings)

    @cached_property
    def rerankings(self) -> RerankingsResourceWithRawResponse:
        return RerankingsResourceWithRawResponse(self._models.rerankings)

    @cached_property
    def completions(self) -> CompletionsResourceWithRawResponse:
        return CompletionsResourceWithRawResponse(self._models.completions)

    @cached_property
    def chat_completions(self) -> ChatCompletionsResourceWithRawResponse:
        return ChatCompletionsResourceWithRawResponse(self._models.chat_completions)

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResourceWithRawResponse:
        return UsageStatisticsResourceWithRawResponse(self._models.usage_statistics)


class AsyncModelsResourceWithRawResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.create = async_to_raw_response_wrapper(
            models.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            models.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            models.update,
        )
        self.list = async_to_raw_response_wrapper(
            models.list,
        )
        self.delete = async_to_raw_response_wrapper(
            models.delete,
        )

    @cached_property
    def deployments(self) -> AsyncDeploymentsResourceWithRawResponse:
        return AsyncDeploymentsResourceWithRawResponse(self._models.deployments)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithRawResponse:
        return AsyncEmbeddingsResourceWithRawResponse(self._models.embeddings)

    @cached_property
    def rerankings(self) -> AsyncRerankingsResourceWithRawResponse:
        return AsyncRerankingsResourceWithRawResponse(self._models.rerankings)

    @cached_property
    def completions(self) -> AsyncCompletionsResourceWithRawResponse:
        return AsyncCompletionsResourceWithRawResponse(self._models.completions)

    @cached_property
    def chat_completions(self) -> AsyncChatCompletionsResourceWithRawResponse:
        return AsyncChatCompletionsResourceWithRawResponse(self._models.chat_completions)

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResourceWithRawResponse:
        return AsyncUsageStatisticsResourceWithRawResponse(self._models.usage_statistics)


class ModelsResourceWithStreamingResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.create = to_streamed_response_wrapper(
            models.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            models.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            models.update,
        )
        self.list = to_streamed_response_wrapper(
            models.list,
        )
        self.delete = to_streamed_response_wrapper(
            models.delete,
        )

    @cached_property
    def deployments(self) -> DeploymentsResourceWithStreamingResponse:
        return DeploymentsResourceWithStreamingResponse(self._models.deployments)

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithStreamingResponse:
        return EmbeddingsResourceWithStreamingResponse(self._models.embeddings)

    @cached_property
    def rerankings(self) -> RerankingsResourceWithStreamingResponse:
        return RerankingsResourceWithStreamingResponse(self._models.rerankings)

    @cached_property
    def completions(self) -> CompletionsResourceWithStreamingResponse:
        return CompletionsResourceWithStreamingResponse(self._models.completions)

    @cached_property
    def chat_completions(self) -> ChatCompletionsResourceWithStreamingResponse:
        return ChatCompletionsResourceWithStreamingResponse(self._models.chat_completions)

    @cached_property
    def usage_statistics(self) -> UsageStatisticsResourceWithStreamingResponse:
        return UsageStatisticsResourceWithStreamingResponse(self._models.usage_statistics)


class AsyncModelsResourceWithStreamingResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.create = async_to_streamed_response_wrapper(
            models.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            models.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            models.update,
        )
        self.list = async_to_streamed_response_wrapper(
            models.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            models.delete,
        )

    @cached_property
    def deployments(self) -> AsyncDeploymentsResourceWithStreamingResponse:
        return AsyncDeploymentsResourceWithStreamingResponse(self._models.deployments)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        return AsyncEmbeddingsResourceWithStreamingResponse(self._models.embeddings)

    @cached_property
    def rerankings(self) -> AsyncRerankingsResourceWithStreamingResponse:
        return AsyncRerankingsResourceWithStreamingResponse(self._models.rerankings)

    @cached_property
    def completions(self) -> AsyncCompletionsResourceWithStreamingResponse:
        return AsyncCompletionsResourceWithStreamingResponse(self._models.completions)

    @cached_property
    def chat_completions(self) -> AsyncChatCompletionsResourceWithStreamingResponse:
        return AsyncChatCompletionsResourceWithStreamingResponse(self._models.chat_completions)

    @cached_property
    def usage_statistics(self) -> AsyncUsageStatisticsResourceWithStreamingResponse:
        return AsyncUsageStatisticsResourceWithStreamingResponse(self._models.usage_statistics)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.models import embedding_create_params
from ...types.embedding_response import EmbeddingResponse
from ...types.parameter_bindings_param import ParameterBindingsParam

__all__ = ["EmbeddingsResource", "AsyncEmbeddingsResource"]


class EmbeddingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return EmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return EmbeddingsResourceWithStreamingResponse(self)

    def create(
        self,
        model_deployment_id: str,
        *,
        texts: List[str],
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EmbeddingResponse:
        """
        ### Description

        Computes the text embeddings for text fragments using the model with the given
        model_deployment_id.

        ### Details

        Users can use this API to execute EMBEDDING type EGP model they have access to.
        To use this API, pass in the `id` of a model returned by the V3 Create Model
        API. An example text embedding request

        ```json
        {
          "texts": ["Please compute my embedding vector", "Another text fragment"]
        }
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `model_deployment_id` but received {model_deployment_id!r}"
            )
        return self._post(
            f"/v4/models/{model_deployment_id}/embeddings",
            body=maybe_transform(
                {
                    "texts": texts,
                    "model_request_parameters": model_request_parameters,
                },
                embedding_create_params.EmbeddingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )


class AsyncEmbeddingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncEmbeddingsResourceWithStreamingResponse(self)

    async def create(
        self,
        model_deployment_id: str,
        *,
        texts: List[str],
        model_request_parameters: ParameterBindingsParam | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EmbeddingResponse:
        """
        ### Description

        Computes the text embeddings for text fragments using the model with the given
        model_deployment_id.

        ### Details

        Users can use this API to execute EMBEDDING type EGP model they have access to.
        To use this API, pass in the `id` of a model returned by the V3 Create Model
        API. An example text embedding request

        ```json
        {
          "texts": ["Please compute my embedding vector", "Another text fragment"]
        }
        ```

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_deployment_id:
            raise ValueError(
                f"Expected a non-empty value for `model_deployment_id` but received {model_deployment_id!r}"
            )
        return await self._post(
            f"/v4/models/{model_deployment_id}/embeddings",
            body=await async_maybe_transform(
                {
                    "texts": texts,
                    "model_request_parameters": model_request_parameters,
                },
                embedding_create_params.EmbeddingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )


class EmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = to_raw_response_wrapper(
            embeddings.create,
        )


class AsyncEmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = async_to_raw_response_wrapper(
            embeddings.create,
        )


class EmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = to_streamed_response_wrapper(
            embeddings.create,
        )


class AsyncEmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = async_to_streamed_response_wrapper(
            embeddings.create,
        )
